import { getBox } from './../';

describe('Collection box', function () {

  var elm1 = null;
  var elm2 = null;

  function createElement() {
    var style = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    var element = document.createElement('div');
    element.className = 'testElement';
    setElementStyle(element, style);
    return element;
  }

  function setElementStyle(element) {
    var style = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    for (var key in style) {
      if (style.hasOwnProperty(key)) {
        element.style[key] = style[key];
      }
    }
  }

  beforeEach(function () {
    elm1 = createElement({
      position: 'absolute',
      width: '100px',
      height: '100px',
      left: '0px',
      top: '0px'
    });
    document.body.appendChild(elm1);

    elm2 = createElement({
      position: 'absolute',
      width: '100px',
      height: '100px',
      left: '200px',
      top: '100px'
    });
    document.body.appendChild(elm2);
  });

  afterEach(function () {
    var test_elements = document.querySelectorAll('.testElement');
    for (var i = 0, j = test_elements.length; i < j; i++) {
      var element = test_elements[i];
      element.parentNode.removeChild(element);
    }
  });

  it('should get properties of HTMLCollection', function () {
    var boxes = document.getElementsByTagName('div');
    var box = getBox(boxes);
    expect(box.left).toEqual(0);
    expect(box.top).toEqual(0);
    expect(box.width).toEqual(300);
    expect(box.height).toEqual(200);
  });

  it('should get properties of NodeList', function () {
    var boxes = document.querySelectorAll('div');
    var box = getBox(boxes);
    expect(box.left).toEqual(0);
    expect(box.top).toEqual(0);
    expect(box.width).toEqual(300);
    expect(box.height).toEqual(200);
  });

  it('should treat array as collection', function () {
    var boxes = [elm1, elm2];
    var box = getBox(boxes);
    expect(box.left).toEqual(0);
    expect(box.top).toEqual(0);
    expect(box.width).toEqual(300);
    expect(box.height).toEqual(200);
  });

  it('should return zero box when not matching any elements', function () {
    var boxes = document.getElementsByTagName('xxx');
    var box = getBox(boxes);
    expect(box.left).toEqual(0);
    expect(box.top).toEqual(0);
    expect(box.width).toEqual(0);
    expect(box.height).toEqual(0);
  });
});