import Box from './element';

describe('Element box', function () {

  var elm = null;

  function createElement() {
    var style = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    var element = document.createElement('div');
    element.className = 'testElement';
    setElementStyle(element, style);
    return element;
  }

  function setElementStyle(element) {
    var style = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    for (var key in style) {
      if (style.hasOwnProperty(key)) {
        element.style[key] = style[key];
      }
    }
  }

  beforeEach(function () {
    elm = createElement({
      width: '100px',
      height: '200px'
    });
    document.body.appendChild(elm);
  });

  afterEach(function () {
    var test_elements = document.querySelectorAll('.testElement');
    for (var i = 0, j = test_elements.length; i < j; i++) {
      var element = test_elements[i];
      element.parentNode.removeChild(element);
    }
  });

  it('should get size', function () {
    var box = new Box(elm);
    expect(box.width).toEqual(100);
    expect(box.height).toEqual(200);
  });

  it('should get position of absolute element', function () {
    setElementStyle(elm, {
      position: 'absolute',
      left: '100px',
      top: '200px'
    });
    var box = new Box(elm);
    expect(box.left).toEqual(100);
    expect(box.top).toEqual(200);
  });

  it('should get position of relative element', function () {
    var wrapper = document.body.appendChild(createElement({
      position: 'absolute',
      left: '0px',
      top: '0px'
    }));

    setElementStyle(elm, {
      position: 'relative',
      left: '100px',
      top: '200px'
    });
    wrapper.appendChild(elm);

    var box = new Box(elm);
    expect(box.left).toEqual(100);
    expect(box.top).toEqual(200);
  });

  it('should get new properties when changes', function () {
    var box = new Box(elm);

    setElementStyle(elm, {
      position: 'absolute',
      left: '1000px',
      top: '2000px',
      width: '3000px',
      height: '4000px'
    });

    box.update();

    expect(box.left).toEqual(1000);
    expect(box.top).toEqual(2000);
    expect(box.width).toEqual(3000);
    expect(box.height).toEqual(4000);
  });

  it('should not throw if element is removed', function () {
    var box = new Box(elm);
    var fn = function fn() {
      box.update();
    };
    elm.parentNode.removeChild(elm);
    expect(fn).not.toThrow();
  });
});