var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Data object representing a box, with basic properties.
 * @typedef {Object} BoxProperties
 * @type Object
 * @property {number} [left]
 * @property {number} [top]
 * @property {number} [right]
 * @property {number} [bottom]
 * @property {number} [width]
 * @property {number} [height]
 */

/**
 * Class representing an abstract Box.
 * @class
 * @name Box
 */
var Box = function () {

  /**
   * Constructs generic Box object.
   * @param {BoxProperties} input
   */
  function Box() {
    var input = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    _classCallCheck(this, Box);

    // default values
    this.left = 0;
    this.top = 0;
    this.width = 0;
    this.height = 0;

    this.input = input;
    this.update();
  }

  _createClass(Box, [{
    key: 'set',


    /**
     * Updates properties of the box
     * @param {BoxProperties} properties
     * @returns {BoxProperties}
     */
    value: function set() {
      var properties = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      if ((typeof properties === 'undefined' ? 'undefined' : _typeof(properties)) === 'object' && properties !== null) {
        var _arr = ['left', 'top', 'width', 'height'];

        for (var _i = 0; _i < _arr.length; _i++) {
          var property = _arr[_i];
          if (typeof properties[property] !== 'undefined') {
            this[property] = properties[property];
          }
        }
      }
      return this.get();
    }

    // returns object with all current properties
    /**
     * Gets current properties of the box.
     * @returns {BoxProperties}
     */

  }, {
    key: 'get',
    value: function get() {
      return {
        left: this.left,
        top: this.top,
        right: this.right,
        bottom: this.bottom,
        width: this.width,
        height: this.height
      };
    }

    // In generic box, this does nothing. It is just a placeholder for method
    // that will be used to get current values for DOM element, collection, etc.
    /**
     * Updates properties of the Box according to the Box's input. This doesn't really do anything in Generic Box. But it recalculates box properties for DOM element, collection, viewport, etc.
     * @returns {BoxProperties}
     */

  }, {
    key: 'update',
    value: function update() {
      return this.set(this.input);
    }

    /**
     * Adds padding to the whole box.
     * @param {number} [padding=0]
     */

  }, {
    key: 'pad',
    value: function pad() {
      var padding = arguments.length <= 0 || arguments[0] === undefined ? 0 : arguments[0];

      this.padHorizontal(padding);
      this.padVertical(padding);
      return this.get();
    }

    /**
     * Adds padding to the left and right side of box.
     * @param {number} [padding=0]
     */

  }, {
    key: 'padHorizontal',
    value: function padHorizontal() {
      var padding = arguments.length <= 0 || arguments[0] === undefined ? 0 : arguments[0];

      this.left -= padding;
      this.width += padding * 2;
      return this.get();
    }

    /**
     * Adds padding to the top and bottom side of box.
     * @param {number} [padding=0]
     */

  }, {
    key: 'padVertical',
    value: function padVertical() {
      var padding = arguments.length <= 0 || arguments[0] === undefined ? 0 : arguments[0];

      this.top -= padding;
      this.height += padding * 2;
      return this.get();
    }

    /**
     * Move box to a specific location
     * @param {number} [left]
     * @param {number} [top]
     * @returns {Box}
     */

  }, {
    key: 'moveTo',
    value: function moveTo(left, top) {
      if (typeof left === 'number') {
        this.left = left;
      }
      if (typeof top === 'number') {
        this.top = top;
      }
      return this.get();
    }

    /**
     * Moves pivot of the box to a specific location, recalculates the other properties accordingly.
     * @param {number} [left]
     * @param {number} [top]
     * @returns {BoxProperties}
     */

  }, {
    key: 'movePivotTo',
    value: function movePivotTo(left, top) {
      if (typeof left !== 'number') {
        left = this.pivot.left;
      }
      if (typeof top !== 'number') {
        top = this.pivot.top;
      }

      this.moveTo(left - this.width / 2, top - this.height / 2);

      return this.get();
    }
  }, {
    key: 'moveBy',


    /**
     * Moves the box by a distance.
     * @param {number} [horizontal]
     * @param {number} [vertical]
     * @returns {BoxProperties}
     */
    value: function moveBy(horizontal, vertical) {
      if (typeof horizontal === 'number') {
        this.left = this.left + horizontal;
      }
      if (typeof vertical === 'number') {
        this.top = this.top + vertical;
      }
      return this.get();
    }

    /**
     * Resizes the box to a specific size.
     * @param [width]
     * @param [height]
     * @returns {BoxProperties}
     */

  }, {
    key: 'resizeTo',
    value: function resizeTo(width, height) {
      if (typeof width === 'number') {
        this.width = width;
      }
      if (typeof height === 'number') {
        this.height = height;
      }
      return this.get();
    }

    /**
     * Resizes the box by a specific value.
     * @param {number} [horizontal]
     * @param {number} [vertical]
     * @returns {BoxProperties}
     */

  }, {
    key: 'resizeBy',
    value: function resizeBy(horizontal, vertical) {
      if (typeof horizontal === 'number') {
        this.width += horizontal;
      }
      if (typeof vertical === 'number') {
        this.height += vertical;
      }
      return this.get();
    }

    /**
     * Returns text representation of Box properties. Used for debugging.
     * @param {boolean} [pretty=false] If `true`, returns pretty formatted string.
     * @returns {string}
     */

  }, {
    key: 'toString',
    value: function toString() {
      var pretty = arguments.length <= 0 || arguments[0] === undefined ? false : arguments[0];

      var indent = pretty === true ? 2 : 0;
      return JSON.stringify(this.get(), null, indent);
    }
  }, {
    key: 'right',
    get: function get() {
      return this.left + this.width;
    }
  }, {
    key: 'bottom',
    get: function get() {
      return this.top + this.height;
    }

    /**
     * @property {object} pivot
     * @property {number} pivot.left - Left position of Box's pivot.
     * @property {number} pivot.top - Top position of Box's pivot.
     */

  }, {
    key: 'pivot',
    get: function get() {
      return {
        left: this.left + this.width / 2,
        top: this.top + this.height / 2
      };
    }
  }]);

  return Box;
}();

export default Box;